package jmsconsumer;

import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author hbragg
 */
public abstract class Task extends Thread {
  private Lock lock = new ReentrantLock();
  protected boolean stopMe = false;
  protected final int processFrequency = 2000;
  protected final int restPeriod = 250;
  protected Logger log;

  public void Task() {
  }
  /**
   * Start this thread. Lock the thread to signal that it is running and
   * using non-java resources which should be released before completion
   */
  @Override
  public void run() {
    log = Logger.getLogger(this.getClass().getSimpleName());
    doNotDisturb();
    try {
      myTask();
    } finally {
      allDone();
    }
  }

  protected abstract void myTask();

  /**
   * Used to cleanup sql connection and then signal waiting
   * threads that this thread has completed its task
   * Should be the last instruction executed in the thread
   */
  private void allDone() {
    lock.unlock();
  }

  /**
   * Used to signal waiting threads that this thread is busy
   * and has non java resources that need to be released
   */
  private void doNotDisturb() {
    lock.lock();
    stopMe = false;
  }

  /**
   * Used to make sure the processor isn't throttled so other threads
   * can access the processor. Short breaks so shutdown is responsive
   */
  protected void rest() {
    for (int i=0; i<processFrequency && !stopMe; i+=restPeriod) {
      try {
        Thread.sleep(restPeriod);
      } catch (InterruptedException ignore) {
        break;
      }
    }
  }

  /**
   * Starts this thread.
   */
  public void startMe() {
    start();
  }

  /**
   * Signals this thread to stop.
   */
  public void stopMe() {
    stopMe = true;
  }

  /**
   * Wait for this thread to complete.
   */
  public void waitMe() {
    try {
      lock.lock();
    } finally {
      lock.unlock();
    }
  }
}
